from rest_framework import serializers


class PaymentSerializer(serializers.Serializer):
    external_id = serializers.CharField(max_length=255)
    amount = serializers.CharField(max_length=255)
    currency = serializers.CharField(max_length=255)
    continue_url = serializers.CharField(max_length=255)
    provider_code = serializers.CharField(max_length=255, required=False,
                                          allow_blank=True)


class NotifySerializer(serializers.Serializer):
    id = serializers.IntegerField(read_only=True)
    name = serializers.CharField(max_length=256)
    timestamp = serializers.DateTimeField()
    payload = serializers.JSONField()



from django.utils.decorators import method_decorator
from django.views.decorators.csrf import csrf_exempt
from rest_framework import status
from rest_framework.response import Response
from rest_framework.views import APIView

from . import signals
from .serializers import NotifySerializer


@method_decorator(csrf_exempt, name='dispatch')
class NotifyView(APIView):

    def post(self, request, format=None):
        serializer = NotifySerializer(data=request.data)
        if serializer.is_valid():
            self._emit_signal(
                serializer.data['name'], serializer.data
            )
            return Response(serializer.data)
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)

    @staticmethod
    def _emit_signal(name, data):
        signals.notification_received.send(
            sender=None,
            name=name, data=data
        )



import setuptools


def version():
    with open('talar/VERSION', 'r') as file:
        return file.read()


with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="django-talar",
    version=version(),
    author="Talar",
    author_email="kamil.obstawski@rakki.xyz",
    description="Django app for [Talar.app](https://talar.app) service.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://bitbucket.org/rakki-software/django-talar/src/",
    packages=setuptools.find_packages(),
    install_requires=[
        'django >=2.1,<2.3', 'djangorestframework >=3.9,<4.0',
        'M2Crypto >=0.30,<0.40',
    ],
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



from django.test import TestCase

from .crypto import AESCipher, AESCipherBase64


class AESCipherTestCase(TestCase):

    BINARY_ENCRYPTION_KEY = b'TEST_BINARY_ENCRYPTION_KEY'
    ENCRYPTION_KEY = 'F5o77VF5ZbsLlUJncgMAtU_cf56WBIORSOyp4-x5wbc===='

    PLAINTEXT = 'To be encrypted 1234.'
    BINARY_DATA = b'TEST_BINARY_DATA'

    def test_aes_cipher_binary_data_encryption_decryption(self):
        cipher = AESCipher(self.BINARY_ENCRYPTION_KEY)
        encrypted = cipher.encrypt(self.BINARY_DATA)
        decrypted = cipher.decrypt(encrypted)
        self.assertEqual(decrypted, self.BINARY_DATA)
        self.assertNotEqual(encrypted, self.BINARY_DATA)

    def test_base64_string_encryption_decryption_flow(self):
        cipher = AESCipherBase64(self.ENCRYPTION_KEY)
        encrypted = cipher.encrypt(self.PLAINTEXT)
        decrypted = cipher.decrypt(encrypted)
        self.assertEqual(decrypted, self.PLAINTEXT)
        self.assertNotEqual(encrypted, self.PLAINTEXT)



from base64 import b64decode, b64encode, urlsafe_b64decode
from enum import IntEnum
from os import urandom

import M2Crypto


class AESOperation(IntEnum):
    """ AES operation IDs """
    DECRYPT = 0
    ENCRYPT = 1


class AESCipher:
    """ Wrapper around M2Crypto to use AES cipher """

    ALGORITHM = 'aes_256_cbc'
    BLOCK_SIZE = 16

    def __init__(self, key: bytes):
        self.key = key

    def encrypt(self, data: bytes) -> bytes:
        """
        Encrypt given string using following scheme:
            {INITIALIZATION_VECTOR}{CIPHERTEXT}
        Where:
            * INITIALIZATION_VECTOR is with size of self.BLOCK_SIZE in bytes
            * CIPHERTEXT is our encrypted data
        """
        initialization_vector = self._generate_iv()
        cipher = self._get_cipher(initialization_vector, AESOperation.ENCRYPT)
        ciphertext = cipher.update(data) + cipher.final()
        return initialization_vector + ciphertext

    def decrypt(self, data: bytes) -> bytes:
        """ Decrypt ciphertext containing IV, returning plaintext. """
        # Extract first {self.BLOCK_SIZE} bytes, containing our IV
        initialization_vector = data[:self.BLOCK_SIZE]
        # Extract ciphertext that is stored after IV
        ciphertext = data[self.BLOCK_SIZE:]
        # Ciphertext decryption using extracted IV
        cipher = self._get_cipher(initialization_vector, AESOperation.DECRYPT)
        return cipher.update(ciphertext) + cipher.final()

    def _get_cipher(self, initialization_vector, operation: AESOperation):
        return M2Crypto.EVP.Cipher(
            self.ALGORITHM, self.key, initialization_vector, operation
        )

    @classmethod
    def _generate_iv(cls) -> bytes:
        return urandom(cls.BLOCK_SIZE)


class AESCipherBase64(AESCipher):
    """ Use AESCipher with base64 encoded keys and inputs """

    def __init__(self, key: str):
        super().__init__(self.b64_decode_with_missing_padding(key))

    def encrypt(self, data: str) -> str:
        """ Encrypt string, returning ciphertext encoded in base64 """
        encrypted = super().encrypt(data.encode('utf-8'))
        return b64encode(encrypted).decode('utf-8')

    def decrypt(self, data: str) -> str:
        """ Returns decrypted plaintext """
        data = b64decode(data)
        return super().decrypt(data).decode('utf-8')

    @staticmethod
    def b64_decode_with_missing_padding(base64_text) -> bytes:
        """ Used for decoding key as its padding is stripped """
        base64_text += '=' * (len(base64_text) % 4)
        return urlsafe_b64decode(base64_text)
